/* BdfServer - Copyright (c) 2012-2023 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.impl;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.api.users.BdfUserConstants;
import fr.exemole.bdfserver.api.users.BdfUserPrefs;
import fr.exemole.bdfserver.tools.BdfServerUtils;
import fr.exemole.bdfserver.tools.configuration.ConfigurationUtils;
import fr.exemole.bdfserver.tools.roles.PermissionSummaryBuilder;
import fr.exemole.bdfserver.tools.subsettree.TreeUtils;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import net.fichotheque.EditOrigin;
import net.fichotheque.corpus.Fiches;
import net.fichotheque.corpus.SortConstants;
import net.fichotheque.permission.PermissionSummary;
import net.fichotheque.selection.FicheQuery;
import net.fichotheque.selection.FicheSelector;
import net.fichotheque.selection.SelectionContext;
import net.fichotheque.sphere.LoginKey;
import net.fichotheque.sphere.Redacteur;
import net.fichotheque.tools.corpus.FichesBuilder;
import net.fichotheque.tools.permission.PermissionUtils;
import net.fichotheque.tools.selection.FicheQueryBuilder;
import net.fichotheque.tools.selection.UserConditionBuilder;
import net.fichotheque.utils.EditOriginUtils;
import net.fichotheque.utils.selection.FicheSelectorBuilder;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LangPreference;
import net.mapeadores.util.localisation.LangPreferenceBuilder;
import net.mapeadores.util.text.TypoOptions;


/**
 *
 * @author Vincent Calame
 */
class BdfUserImpl implements BdfUser {

    private static long lastBdfSessionId = 0;
    private final Map<String, Object> parameterMap = new HashMap<String, Object>();
    private final String bdfServerName;
    private final LoginKey loginKey;
    private final Redacteur redacteur;
    private final BdfUserPrefsImpl prefs;
    private FicheQuery ficheQuery;
    private String sortType = SortConstants.NONE;
    private Fiches selectedFiches = null;
    private boolean ficheSelectionCustomized = false;
    private final String bdfSessionId;
    private Lang workingLang;
    private Locale formatLocale;
    private LangPreference langPreference;

    private BdfUserImpl(String bdfServerName, Redacteur redacteur) {
        this.bdfServerName = bdfServerName;
        this.loginKey = LoginKey.build(redacteur.getSubsetKey(), redacteur.getLogin());
        this.bdfSessionId = getNewBdfSessionId();
        this.redacteur = redacteur;
        this.prefs = new BdfUserPrefsImpl(this);
        this.ficheQuery = FicheQueryBuilder.init()
                .setUserCondition(UserConditionBuilder.init().addId(redacteur).toUserCondition())
                .toFicheQuery();
        prefs.defaultFicheQuery = ficheQuery;
    }

    void updateLocalisation(BdfServer bdfServer) {
        workingLang = prefs.getWorkingLang();
        if (workingLang == null) {
            workingLang = ConfigurationUtils.getDefaultWorkingLang(bdfServer.getLangConfiguration());
        }
        formatLocale = prefs.getCustomFormatLocale();
        if (formatLocale == null) {
            formatLocale = workingLang.toLocale();
        }
        putParameter(BdfUserConstants.FORM_TYPOOPTIONS, TypoOptions.getTypoOptions(formatLocale));
        LangPreferenceBuilder langPreferenceBuilder = new LangPreferenceBuilder();
        LangPreference customLangPreference = prefs.getCustomLangPreference();
        if (customLangPreference != null) {
            langPreferenceBuilder.addLangs(customLangPreference);
        }
        langPreferenceBuilder.addLang(workingLang);
        ConfigurationUtils.checkLangPreference(bdfServer.getLangConfiguration(), langPreferenceBuilder);
        this.langPreference = langPreferenceBuilder.toLangPreference();
    }

    @Override
    public LoginKey getLoginKey() {
        return loginKey;
    }

    @Override
    public String getBdfServerName() {
        return bdfServerName;
    }

    @Override
    public Redacteur getRedacteur() {
        return redacteur;
    }

    @Override
    public BdfUserPrefs getPrefs() {
        return prefs;
    }

    @Override
    public Lang getWorkingLang() {
        return workingLang;
    }

    @Override
    public Locale getFormatLocale() {
        return formatLocale;
    }

    @Override
    public LangPreference getLangPreference() {
        return langPreference;
    }

    @Override
    public FicheQuery getFicheQuery() {
        return ficheQuery;
    }

    @Override
    public String getSortType() {
        return sortType;
    }

    @Override
    public Fiches getSelectedFiches() {
        return selectedFiches;
    }

    @Override
    public void setSelectedFiches(Fiches fiches) {
        this.selectedFiches = fiches;
        this.ficheSelectionCustomized = true;
    }

    @Override
    public Object getParameterValue(String parameterName) {
        return parameterMap.get(parameterName);
    }

    @Override
    public void putParameter(String parameterName, Object parameterValue) {
        parameterMap.put(parameterName, parameterValue);
    }

    @Override
    public EditOrigin newEditOrigin(String source) {
        return EditOriginUtils.newEditOrigin(bdfSessionId, redacteur.getGlobalId(), source);
    }

    void setFicheQuery(FicheQuery ficheQuery) {
        if (ficheQuery == null) {
            throw new IllegalArgumentException("ficheQuery is null");
        }
        this.ficheQuery = ficheQuery;
        this.ficheSelectionCustomized = false;
    }

    void setFicheSelection(BdfServer bdfServer, FicheQuery ficheQuery, String sortType) {
        if (ficheQuery == null) {
            throw new IllegalArgumentException("ficheQuery is null");
        }
        this.ficheQuery = ficheQuery;
        this.sortType = sortType;
        this.ficheSelectionCustomized = false;
        updateFicheSelection(bdfServer);
    }

    boolean updateFicheSelection(BdfServer bdfServer) {
        if (ficheSelectionCustomized) {
            return false;
        }
        PermissionSummary permissionSummary = PermissionSummaryBuilder.build(bdfServer, this);
        SelectionContext selectionContext = BdfServerUtils.initSelectionContextBuilder(bdfServer, getWorkingLang())
                .setSubsetAccessPredicate(permissionSummary.getSubsetAccessPredicate())
                .setFichePredicate(PermissionUtils.getFichePredicate(permissionSummary))
                .toSelectionContext();
        FicheSelector ficheSelector = FicheSelectorBuilder.init(selectionContext).add(getFicheQuery()).toFicheSelector();
        this.selectedFiches = FichesBuilder.build(sortType, getWorkingLang())
                .initSubsetKeyOrder(TreeUtils.getCorpusKeyList(bdfServer))
                .populate(ficheSelector)
                .toFiches();
        return true;
    }

    private void initBdfUserPrefs(BdfUserPrefs bdfUserPrefs) {
        prefs.init(bdfUserPrefs);
        FicheQuery defaultFicheQuery = bdfUserPrefs.getDefaultFicheQuery();
        if (defaultFicheQuery != null) {
            this.ficheQuery = defaultFicheQuery;
            this.sortType = bdfUserPrefs.getDefaultSortType();
            this.ficheSelectionCustomized = false;
        }
    }

    private synchronized static String getNewBdfSessionId() {
        long l = System.currentTimeMillis();
        if (l <= lastBdfSessionId) {
            l = lastBdfSessionId + 1;
        }
        lastBdfSessionId = l;
        return String.valueOf(l);
    }

    static BdfUserImpl build(BdfServer bdfServer, Redacteur redacteur) {
        BdfUserImpl bdfUser = new BdfUserImpl(bdfServer.getName(), redacteur);
        BdfUserPrefs storagePrefs = bdfServer.getBdfUserStorage().getBdfUserPrefs(redacteur);
        if (storagePrefs != null) {
            bdfUser.initBdfUserPrefs(storagePrefs);
        }
        bdfUser.updateLocalisation(bdfServer);
        return bdfUser;
    }

}
