/* BdfServer_Commands - Copyright (c) 2023 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.corpus;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.BdfServerEditor;
import fr.exemole.bdfserver.api.EditSession;
import fr.exemole.bdfserver.api.externalsource.ExternalSourceType;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.api.ui.DataUi;
import fr.exemole.bdfserver.api.ui.UiComponents;
import fr.exemole.bdfserver.tools.externalsource.ExternalSourceUtils;
import fr.exemole.bdfserver.tools.instruction.AbstractBdfCommand;
import fr.exemole.bdfserver.tools.instruction.BdfCommandUtils;
import fr.exemole.bdfserver.tools.instruction.BdfErrors;
import fr.exemole.bdfserver.tools.ui.components.DataUiBuilder;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.tools.externalsource.ExternalSourceDefBuilder;
import net.fichotheque.tools.parsers.TypoParser;
import net.mapeadores.util.attr.AttributeChange;
import net.mapeadores.util.attr.AttributeParser;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.logging.ErrorMessageException;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.CleanedString;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.LabelChange;
import net.mapeadores.util.text.LabelUtils;
import net.mapeadores.util.text.TypoOptions;


/**
 *
 * @author Vincent Calame
 */
public class DataChangeCommand extends AbstractBdfCommand {

    public final static String COMMANDNAME = "DataChangeCommand";
    public final static String COMMANDKEY = "_ CRP-34";
    public final static String NAME_PARAMNAME = "name";
    public final static String TITLE_PARAMPREFIX = "title/";
    public final static String PARAM_PARAMPREFIX = "param_";
    public final static String ATTRIBUTES_PARAMNAME = "attributes";
    private Corpus corpus;
    private UiComponents uiComponents;
    private DataUi dataUi;
    private LabelChange labelChange;
    private AttributeChange attributeChange;

    public DataChangeCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return true;
    }

    @Override
    protected void doCommand() {
        boolean done;
        try (EditSession session = startEditSession(Domains.CORPUS, COMMANDNAME)) {
            done = updateDataUi(session.getBdfServerEditor(), dataUi);
        }
        putResultObject(CORPUS_OBJ, corpus);
        if (done) {
            setDone("_ done.corpus.datachange", dataUi.getDataName());
        }
    }

    @Override
    protected void checkParameters() throws ErrorMessageException {
        corpus = getMandatoryCorpus();
        checkSubsetAdmin(corpus);
        String name = getMandatory(NAME_PARAMNAME);
        uiComponents = bdfServer.getUiManager().getMainUiComponents(corpus);
        dataUi = (DataUi) uiComponents.getUiComponent(DataUi.toComponentName(name));
        if (dataUi == null) {
            throw BdfErrors.unknownParameterValue(NAME_PARAMNAME, name);
        }
        this.labelChange = BdfCommandUtils.getLabelChange(requestMap, TITLE_PARAMPREFIX, false);
        String attributes = getMandatory(ATTRIBUTES_PARAMNAME);
        this.attributeChange = AttributeParser.parse(attributes);
    }

    private boolean updateDataUi(BdfServerEditor bdfServerEditor, DataUi dataUi) {
        DataUiBuilder dataUiBuilder = DataUiBuilder.derive(dataUi);
        for (Label label : labelChange.getChangedLabels()) {
            Lang currentLang = label.getLang();
            TypoOptions typoOptions = TypoOptions.getTypoOptions(currentLang.toLocale());
            String labelString = label.getLabelString();
            String checkedLabelString = TypoParser.parseTypo(labelString, typoOptions);
            if (!checkedLabelString.equals(labelString)) {
                CleanedString cleanedString = CleanedString.newInstance(checkedLabelString);
                if (cleanedString == null) {
                    dataUiBuilder.removeLabel(currentLang);
                    continue;
                } else {
                    label = LabelUtils.toLabel(currentLang, cleanedString);
                }
            }
            dataUiBuilder.putLabel(label);
        }
        for (Lang lang : labelChange.getRemovedLangList()) {
            dataUiBuilder.removeLabel(lang);
        }
        ExternalSourceType externalSourceType = ExternalSourceUtils.getMatchingExternalSourceType(dataUi.getExternalSourceDef());
        ExternalSourceDefBuilder externalSourceDefBuilder = dataUiBuilder.getExternalSourceDefBuilder();
        for (ExternalSourceType.Param param : externalSourceType.getParamList()) {
            String paramName = param.getName();
            String value = requestMap.getParameter(PARAM_PARAMPREFIX + paramName);
            externalSourceDefBuilder.addParam(paramName, value);
        }
        dataUiBuilder.changesAttribute(attributeChange);
        bdfServerEditor.putComponentUi(uiComponents, dataUiBuilder.toDataUi());
        return true;
    }

}
