/* BdfServer_Commands - Copyright (c) 2012-2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.commands.exportation;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.managers.SqlExportManager;
import fr.exemole.bdfserver.tools.instruction.AbstractBdfCommand;
import fr.exemole.bdfserver.tools.instruction.BdfCommandUtils;
import fr.exemole.bdfserver.tools.instruction.BdfErrors;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import java.text.ParseException;
import net.fichotheque.exportation.sql.SqlExportDef;
import net.fichotheque.tools.exportation.sql.SqlExportDefBuilder;
import net.fichotheque.tools.selection.SelectionDOMUtils;
import net.mapeadores.util.attr.AttributeParser;
import net.mapeadores.util.logging.ErrorMessageException;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.LabelChange;
import net.mapeadores.util.text.RelativePath;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.xml.DOMUtils;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;


/**
 *
 * @author Vincent Calame
 */
public class SqlExportChangeCommand extends AbstractBdfCommand {

    public final static String COMMANDNAME = "SqlExportChange";
    public final static String COMMANDKEY = "_ EXP-43";
    public final static String CUSTOMCLASSNAME_PARAMNAME = "customclassname";
    public final static String PARAMS_PARAMNAME = "params";
    public final static String TARGETNAME_PARAMNAME = "targetname";
    public final static String TARGETPATH_PARAMNAME = "targetpath";
    public final static String FILENAME_PARAMNAME = "filename";
    public final static String TABLEEXPORTNAME_PARAMNAME = "tableexportname";
    public final static String POSTCOMMAND_PARAMNAME = "postcommand";
    public final static String QUERY_XML_PARAMNAME = "query_xml";
    public final static String TITLE_PARAMPREFIX = "title/";
    public final static String ATTRIBUTES_PARAMNAME = "attributes";
    private SqlExportDef sqlExportDef;

    public SqlExportChangeCommand(BdfServer bdfServer, RequestMap requestMap) {
        super(bdfServer, requestMap);
    }

    @Override
    public boolean needSynchronisation() {
        return true;
    }

    @Override
    protected void doCommand() {
        SqlExportManager sqlExportManager = bdfServer.getSqlExportManager();
        sqlExportManager.putSqlExportDef(sqlExportDef);
        putResultObject(SQLEXPORTDEF_OBJ, sqlExportDef);
        setDone("_ done.exportation.sqlexportchange", sqlExportDef.getName());
    }

    @Override
    protected void checkParameters() throws ErrorMessageException {
        checkFichothequeAdmin();
        SqlExportManager sqlExportManager = bdfServer.getSqlExportManager();
        SqlExportDef currentSqlExportDef = BdfInstructionUtils.getMandatorySqlExportDef(sqlExportManager, requestMap);
        String sqlExportClassName = getMandatory(CUSTOMCLASSNAME_PARAMNAME);
        String tableExportName = getMandatory(TABLEEXPORTNAME_PARAMNAME);
        String postCommand = getMandatory(POSTCOMMAND_PARAMNAME);
        String targetName = getMandatory(TARGETNAME_PARAMNAME);
        String targetPathString = getMandatory(TARGETPATH_PARAMNAME);
        String fileName = getMandatory(FILENAME_PARAMNAME);
        RelativePath targetPath;
        try {
            targetPath = RelativePath.parse(targetPathString);
        } catch (ParseException pe) {
            throw BdfErrors.error("_ error.wrong.relativepath", targetPathString);
        }
        SqlExportDefBuilder sqlExportDefBuilder = SqlExportDefBuilder.init(currentSqlExportDef.getName(), currentSqlExportDef.getAttributes())
                .setSqlExportClassName(sqlExportClassName)
                .setPostCommand(postCommand)
                .setTableExportName(tableExportName)
                .setTargetName(targetName)
                .setTargetPath(targetPath);
        try {
            sqlExportDefBuilder.setFileName(fileName);
        } catch (ParseException pe) {
            throw BdfErrors.error("_ error.wrong.filename", fileName);
        }
        String params = getMandatory(PARAMS_PARAMNAME);
        String[] tokens = StringUtils.getLineTokens(params, StringUtils.EMPTY_EXCLUDE);
        for (String token : tokens) {
            int idx = token.indexOf('=');
            if (idx == -1) {
                continue;
            }
            String name = token.substring(0, idx).trim();
            if (name.length() > 0) {
                sqlExportDefBuilder.addParam(name, token.substring(idx + 1));
            }
        }
        String queryXml = getMandatory(QUERY_XML_PARAMNAME);
        queryXml = queryXml.trim();
        if (queryXml.length() > 0) {
            queryXml = "<queries>" + queryXml + "</queries>";
            try {
                Document document = DOMUtils.parseDocument(queryXml);
                DOMUtils.readChildren(document.getDocumentElement(), SelectionDOMUtils.getQueryElementConsumer(fichotheque, sqlExportDefBuilder.getCustomFichothequeQueriesBuilder()));
            } catch (SAXException saxException) {
                throw BdfErrors.error("_ error.wrong.queryxml", saxException.getMessage());
            }
        }
        LabelChange labelChange = BdfCommandUtils.getLabelChange(requestMap, TITLE_PARAMPREFIX);
        for (Label label : labelChange.getChangedLabels()) {
            sqlExportDefBuilder.putLabel(label);
        }
        String attributes = requestMap.getParameter(ATTRIBUTES_PARAMNAME);
        if (attributes != null) {
            sqlExportDefBuilder.getAttributesBuilder().changeAttributes(AttributeParser.parse(attributes));
        }
        this.sqlExportDef = sqlExportDefBuilder.toSqlExportDef();
    }

}
