/* BdfServer_Get - Copyright (c) 2013-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.get.streamproducers;

import fr.exemole.bdfserver.html.BdfServerHtmlProducer;
import java.text.DecimalFormatSymbols;
import java.util.Collection;
import net.fichotheque.Subset;
import net.fichotheque.SubsetItem;
import net.fichotheque.exportation.table.CellEngine;
import net.fichotheque.exportation.table.ColDef;
import net.fichotheque.exportation.table.SubsetTable;
import net.fichotheque.exportation.table.TableExportConstants;
import net.fichotheque.exportation.table.TableExportContext;
import net.fichotheque.format.SourceLabelProvider;
import net.fichotheque.tools.exportation.table.TableExportEngine;
import net.fichotheque.utils.TableDefUtils;
import net.mapeadores.util.html.HtmlPrinter;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.money.Amount;
import net.mapeadores.util.primitives.Decimal;
import net.mapeadores.util.primitives.FuzzyDate;
import net.mapeadores.util.table.TableWriter;
import fr.exemole.bdfserver.api.instruction.BdfParameters;


/**
 *
 * @author Vincent Calame
 */
public class TableExportHtmlProducer extends BdfServerHtmlProducer {

    private final SubsetTable subsetTable;
    private final Collection<SubsetItem> subsetItems;
    private final short headerType;
    private final boolean fragment;
    private final TableExportContext tableExportContext;
    private final CellEngine cellEngine;

    public TableExportHtmlProducer(BdfParameters bdfParameters, SubsetTable subsetTable, Collection<SubsetItem> subsetItems, short headerType, boolean fragment, TableExportContext tableExportContext, CellEngine cellEngine) {
        super(bdfParameters);
        this.subsetTable = subsetTable;
        this.subsetItems = subsetItems;
        this.headerType = headerType;
        this.fragment = fragment;
        this.tableExportContext = tableExportContext;
        this.cellEngine = cellEngine;
        setReversePath("../");
    }

    @Override
    public void printHtml() {
        if (!fragment) {
            start();
        }
        this
                .TABLE();
        Subset subset = subsetTable.getSubset();
        if (headerType == TableExportConstants.COLUMNTITLE_HEADER) {
            printHeaderTitle(subsetTable, workingLang, subset);
        } else if (headerType == TableExportConstants.COLUMNKEY_HEADER) {
            printHeaderKey(subsetTable);
        }
        HtmlSubsetTableWriter htmlSubsetTableWriter = new HtmlSubsetTableWriter(this, new DecimalFormatSymbols(bdfUser.getFormatLocale()));
        TableExportEngine.exportSubset(subsetTable, htmlSubsetTableWriter, cellEngine, subsetItems);
        this
                ._TABLE();
        if (!fragment) {
            end();
        }
    }

    private void printHeaderTitle(SubsetTable table, Lang workingLang, Subset subset) {
        SourceLabelProvider sourceLabelProvider = tableExportContext.getSourceLabelProvider();
        this
                .TR();
        for (ColDef colDef : table.getColDefList()) {
            this
                    .TH()
                    .__escape(TableDefUtils.getColTitle(colDef, workingLang, sourceLabelProvider, subset))
                    ._TH();
        }
        this
                ._TR();
    }

    private void printHeaderKey(SubsetTable table) {
        this
                .TR();
        for (ColDef colDef : table.getColDefList()) {
            this
                    .TH()
                    .__escape(colDef.getColName())
                    ._TH();
        }
        this
                ._TR();
    }


    private static class HtmlSubsetTableWriter implements TableWriter {

        private final HtmlPrinter hp;
        private final DecimalFormatSymbols symbols;
        private int rowNumber;
        private int columnNumber;

        private HtmlSubsetTableWriter(HtmlPrinter hp, DecimalFormatSymbols symbols) {
            this.hp = hp;
            this.symbols = symbols;
            this.rowNumber = 0;
        }

        @Override
        public int startRow() {
            rowNumber++;
            hp
                    .TR();
            columnNumber = 0;
            return rowNumber;
        }

        @Override
        public int addIntegerCell(Long lg) {
            columnNumber++;
            if (lg == null) {
                hp
                        .TD()
                        ._TD();
            } else {
                hp
                        .TD()
                        .__append(lg)
                        ._TD();
            }
            return columnNumber;
        }

        @Override
        public int addDecimalCell(Decimal decimal) {
            columnNumber++;
            if (decimal == null) {
                hp
                        .TD()
                        ._TD();
            } else {
                hp
                        .TD()
                        .__escape(decimal)
                        ._TD();
            }
            return columnNumber;
        }

        @Override
        public int addStringCell(String s) {
            columnNumber++;
            if (s == null) {
                hp
                        .TD()
                        ._TD();
            } else {
                hp
                        .TD();
                int length = s.length();
                for (int i = 0; i < length; i++) {
                    char carac = s.charAt(i);
                    switch (carac) {
                        case '\r':
                            if (i < (length - 1)) {
                                if (s.charAt(i + 1) == '\n') {
                                    i++;
                                }
                            }
                        case '\n':
                            hp
                                    .BR();
                            break;
                        default:
                            hp
                                    .__escape(carac);
                    }
                }
                hp
                        ._TD();
            }
            return columnNumber;
        }

        @Override
        public int addDateCell(FuzzyDate date) {
            columnNumber++;
            if (date == null) {
                hp
                        .TD()
                        ._TD();
            } else {
                hp
                        .TD()
                        .__escape(date.toISOString())
                        ._TD();
            }
            return columnNumber;
        }

        @Override
        public int addMoneyCell(Amount amount) {
            columnNumber++;
            if (amount == null) {
                hp
                        .TD()
                        ._TD();
            } else {
                hp
                        .TD()
                        .__escape(amount.toLitteralString(symbols, false))
                        ._TD();
            }
            return columnNumber;
        }

        @Override
        public int addPercentageCell(Decimal decimal) {
            columnNumber++;
            if (decimal == null) {
                hp
                        .TD()
                        ._TD();
            } else {
                hp
                        .TD()
                        .__escape(decimal)
                        ._TD();
            }
            return columnNumber;
        }

        @Override
        public int endRow() {
            hp
                    ._TR();
            return rowNumber;
        }

    }

}
